<?php
class ControllerExtensionPaymentClickuz extends Controller {

	public function index() {

        $this->load->language('extension/payment/clickuz');
        $this->load->model('checkout/order');

        $order_info                 = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $data['button_confirm']     = $this->language->get('button_confirm');
        $data['action']             = 'https://my.click.uz/pay/';

        $data['merchantID']         = $this->config->get('payment_clickuz_merchant_id');
        $data['merchantUserID']     = $this->config->get('payment_clickuz_merchant_user_id');
        $data['merchantServiceID']  = $this->config->get('payment_clickuz_service_id');
        $data['transID']            = $this->session->data['order_id'];
        $data['transAmount']        = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
        $data['transNote']          = '';
        $data['signTime']           = date("Y-m-d h:i:s");;
        $data['signString']         = md5 ($data['signTime']. $this->config->get('payment_clickuz_secret_key'). $data['merchantServiceID']. $data['transID']. $data['transAmount']);

        $data['clickuz_method']     = $this->config->get('payment_clickuz_method');

        $data['redirectUrl']        = $this->url->link('extension/payment/clickuz/pay');

        return $this->load->view('extension/payment/clickuz', $data);
	}

	public function pay() {
	    // https://my.click.uz/services/pay?service_id={{merchantServiceID}}&merchant_id={{merchantID}}&amount={{transAmount}}&transaction_param={{transID}}&source=test&return_url={{returnURL}}

        if ($this->session->data['payment_method']['code'] == 'clickuz') {
            $this->load->model('checkout/order');

            $order_info     = $this->model_checkout_order->getOrder($this->session->data['order_id']);

            $query_vars = array(
                'service_id'        => $this->config->get('payment_clickuz_service_id'),
                'merchant_id'       => $this->config->get('payment_clickuz_merchant_user_id'),
                'amount'            => $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false),
                'transaction_param' => $this->session->data['order_id'],
                'source'            => 'opencart',
                'return_url'        => $this->url->link('extension/payment/clickuz/callback')
            );

            $pay_url = 'https://my.click.uz/services/pay?' . http_build_query( $query_vars, '', '&');

            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], 1);

            header('Location: '.$pay_url);

            exit;
        }

    }

	public function payment_received() {

        $this->load->model('checkout/order');

        $json = array();

        if ($this->session->data['payment_method']['code'] == 'clickuz') {
            $this->load->model('checkout/order');

            $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('payment_clickuz_order_status_id'));

            $json['redirect'] = $this->url->link('checkout/success');
        }

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

	public function callback() {

	    $redirect = $this->url->link('checkout/success');

	    if( isset($this->request->get['payment_status']) ) {
	        switch ($this->request->get['payment_status']) {
                case 0:
                    $redirect = $this->url->link('checkout/success');
                    break;
                case -10000:
                    $redirect = $this->url->link('checkout/failure');
            }
        }
        $this->response->redirect( $redirect);
    }

    public function prepare() {

	    error_reporting(0);

	    $result = $this->validate_prepare();

        $this->log->write($result);

        $this->model_extension_payment_clickuz->updateLog($result['log_id'], array( 'result_string' => json_encode($result['data']) ) );

        echo json_encode( $result['data'] );

	    exit;
    }

    public function complete() {

	    error_reporting(0);

        $result = $this->validate_complete();

        $this->model_extension_payment_clickuz->updateLog($result['log_id'], array( 'result_string' => json_encode($result['data']) ) );

        echo json_encode( $result['data'] );

        exit;
    }

    public function validate_prepare() {

	    $this->load->model('extension/payment/clickuz');
        $this->load->model('checkout/order');

        $result = array();

        $this->log->write('validate prepare');

        $prepare_id = $this->model_extension_payment_clickuz->addLog($_POST);

        $this->log->write('prepare_id=' . $prepare_id);

	    $result['log_id'] = $prepare_id;

        if( ! isset(
            $_POST['click_trans_id'],
            $_POST['service_id'],
            $_POST['merchant_trans_id'],
            $_POST['amount'],
            $_POST['action'],
            $_POST['sign_time']) ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-8',
                    'error_note' => 'Error in request from click'
                )
            );
        }


        $this->log->write('params check passed');

        $signString = $_POST['click_trans_id'] .
            $_POST['service_id'] .
            $this->config->get('payment_clickuz_secret_key') .
            $_POST['merchant_trans_id'] .
            $_POST['amount'] .
            $_POST['action'] .
            $_POST['sign_time'];

        $signString = md5( $signString );

        if ( $signString !== $_POST['sign_string'] ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-1',
                    'error_note' => 'Sign check error'
                )
            );

        }

        $this->log->write('sign check passed');

        $order_info = $this->model_checkout_order->getOrder($_POST['merchant_trans_id']);

        if( ! $order_info ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-5',
                    'error_note' => 'User does not exist'
                )
            );
        }

        $this->log->write('order check passed');

        $status = $order_info['order_status_id'];

        $paid_statuses = array_merge($this->config->get('config_processing_status'), $this->config->get('config_complete_status'));

        $total = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

        if( in_array($status, $paid_statuses))  {

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-4',
                    'error_note' => 'Already paid'
                )
            );
        }

        $this->log->write('order paid check passed');

        if( abs($total - (float)$_POST['amount']) > 0.01 ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-2',
                    'error_note' => 'Incorrect parameter amount'
                )
            );
        }

        $this->log->write('amount check passed');

        try{
            $this->model_checkout_order->addOrderHistory($order_info['order_id'], $status, 'Prepare request from CLICK processed. CLICK Transaction ID: ' . $_POST['click_trans_id']);

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'click_trans_id' => $_POST['click_trans_id'],
                    'merchant_trans_id' => $_POST['merchant_trans_id'],
                    'merchant_prepare_id' => $prepare_id,
                    'error'      => '0',
                    'error_note' => 'Success'
                )
            );
        } catch (Exception $ex) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-7',
                    'error_note' => 'Failed to update user'
                )
            );
        }

    }

    public function validate_complete() {

        $this->load->model('extension/payment/clickuz');
        $this->load->model('checkout/order');
        $result = array();
        $prepare_id = $this->model_extension_payment_clickuz->addLog($_POST);

        $result['log_id'] = $prepare_id;

        if( ! isset(
            $_POST['click_trans_id'],
            $_POST['service_id'],
            $_POST['merchant_trans_id'],
            $_POST['merchant_prepare_id'],
            $_POST['amount'],
            $_POST['action'],
            $_POST['sign_time']) ) {

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-8',
                    'error_note' => 'Error in request from click'
                )
            );
        }

        $signString = $_POST['click_trans_id'] .
            $_POST['service_id'] .
            $this->config->get('payment_clickuz_secret_key') .
            $_POST['merchant_trans_id'] .
            $_POST['merchant_prepare_id'] .
            $_POST['amount'] .
            $_POST['action'] .
            $_POST['sign_time'];

        $signString = md5( $signString );

        if ( $signString !== $_POST['sign_string'] ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-1',
                    'error_note' => 'Sign check error'
                )
            );
        }

        $order_info = $this->model_checkout_order->getOrder($_POST['merchant_trans_id']);

        if( ! $order_info ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-5',
                    'error_note' => 'User does not exist'
                )
            );
        }

        $status = $order_info['order_status_id'];

        $total = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

        if( ! $this->model_extension_payment_clickuz->getLog($_POST['merchant_prepare_id']) ) {

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-6',
                    'error_note' => 'Transaction does not exist'
                )
            );
        }

        if( abs($total - (float)$_POST['amount']) > 0.01 ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-2',
                    'error_note' => 'Incorrect parameter amount'
                )
            );
        }

        if( $status == 10 ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-9',
                    'error_note' => 'Transaction cancelled'
                )
            );
        }

        if( in_array($status,  array_merge($this->config->get('config_processing_status'), $this->config->get('config_complete_status'))) ) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-4',
                    'error_note' => 'Already paid'
                )
            );
        }

        if( $_POST['error'] < 0 ) {

            $this->model_checkout_order->addOrderHistory($order_info['order_id'], 10, $_POST['error_note']);

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'click_trans_id' => $_POST['click_trans_id'],
                    'merchant_trans_id' => $_POST['merchant_trans_id'],
                    'merchant_confirm_id' => $_POST['merchant_prepare_id'],
                    'error'      => '-9',
                    'error_note' => 'Transaction cancelled'
                )
            );
        }

        try {
            $this->model_checkout_order->addOrderHistory($order_info['order_id'], 2, $_POST['error_note']);

            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'click_trans_id' => $_POST['click_trans_id'],
                    'merchant_trans_id' => $_POST['merchant_trans_id'],
                    'merchant_confirm_id' => $_POST['merchant_prepare_id'],
                    'error'      => '0',
                    'error_note' => 'Success'
                )
            );
        } catch (Exception $ex) {
            return array(
                'log_id' => $prepare_id,
                'data' => array(
                    'error'      => '-7',
                    'error_note' => 'Failed to update user'
                )
            );
        }
    }
}